'use strict';

angular.module('adf.widget.electre.lavoisier')
    .controller('lavoisierBaseEditCtrl',
        [
            "$scope", "config",
            function ($scope, config) {
                config.viewOk = false;
                $scope.select_view = {
                    controller : {},
                    event : {
                        object : undefined,
                        isAdded : false
                    },
                    config : config
                };

                $scope.parameters_view = {
                    controller : {},
                    selectViewControl : $scope.select_view,
                    config : config
                };
            }
        ]
    ).controller('lavoisierGeneriqueEditCtrl',
         [
             "$scope", "$controller", "config", "CONTROLLER_TYPE",
             function ($scope, $controller, config, CONTROLLER_TYPE) {
                 switch (config.type) {
                     case CONTROLLER_TYPE.html:
                         $controller('lavoisierBaseEditCtrl', {
                             $scope : $scope,
                             config : config
                         });
                         break;
                     case CONTROLLER_TYPE.iFrame:
                         $controller('lavoisierIFrameEditCtrl', {
                             $scope : $scope,
                             config : config
                         });
                         break;
                     case CONTROLLER_TYPE.table:
                         $controller('lavoisierBaseEditCtrl', {
                             $scope : $scope,
                             config : config
                         });
                         break;
                     case CONTROLLER_TYPE.text:
                         $controller('lavoisierTextEditCtrl',
                             {
                                 $scope : $scope,
                                 config : config
                             }
                         );
                         break;
                 }
             }
         ]
    ).factory('renderingFactory',
         [
             'domService',
             function (domService) {
                 function domForRenderingSection () {
                     /**
                      * Service de manipulation du DOM.
                      * @type domService
                      */
                     var domService_ = domService;
                     /**
                      *
                      * @param {AngularFactory} factoryRendering
                      * @param {Element} elementToAppendDom
                      * @param {Object} config
                      */
                     this.createRenderingDomFromArray = function (factoryRendering, elementToAppendDom, config) {
                         /**
                          * Label pour l'Element de formulaire.
                          * @type Element
                          */
                         var label = domService_.createElement("label").setAttribute('for', 'select_rendering_for_edit_view').getDom();
                         /**
                          * Select pour le choix du rendering.
                          * @type Element
                          */
                         var select = domService_.createElement("select")
                                 .setAttribute('id', 'select_rendering_for_edit_view')
                                 .setAttribute('class', 'form-control')
                                 .setAttribute('ng-model', 'config.rendering').getDom();
                         /**
                          * Div utilisé pour contenir le 'label' et le 'select'.
                          * @type Element
                          */
                         var div = domService_.createElement("div").setAttribute('class', 'form-group').appendChild(label).appendChild(select).getDom();

                         label.innerHTML = "Rendering";

                         for (var i = 0 ; i < factoryRendering.renderings.length ; i++) {
                             var option = domService_.createElement("option").setAttribute('value', factoryRendering.renderings[i]).getDom();
                             option.innerHTML = factoryRendering.renderings[i];

                             select.appendChild(option);
                         }

                         config.rendering = config.rendering || factoryRendering.defaultRendering;
                         elementToAppendDom.append(div);
                     };
                 }

                 return new domForRenderingSection();
             }
         ]
    ).directive(
         'selectView',
         [

             "lavoisierHttpRequestService", "parserService", "LAVOISIER_TEMPLATES_PATH",
             function (lavoisierHttpRequestService, parserService, LAVOISIER_TEMPLATES_PATH) {
         // Url liste des vues.
         // Par nom :  "http://localhost:8080/lavoisier/configuration/cfg:config/cfg:view[cfg:info/cfg:category!='Hidden'%20and%20cfg:info/cfg:category!='Configuration'%20and%20cfg:info/cfg:category!='Environment'%20and%20cfg:info/cfg:category!='Monitoring']/@name?accept=xml";
         //
         // Par nom et par category :
         // "http://localhost:8080/lavoisier/configuration
         // /cfg:config/cfg:view[cfg:info/cfg:category!='Hidden'%20and%20cfg:info/cfg:category!='Configuration'%20and%20cfg:info/cfg:category!='Environment'%20and%20cfg:info/cfg:category!='Monitoring']/@name|
         // /cfg:config/cfg:view[cfg:info/cfg:category!='Hidden'%20and%20cfg:info/cfg:category!='Configuration'%20and%20cfg:info/cfg:category!='Environment'%20and%20cfg:info/cfg:category!='Monitoring']/cfg:info/cfg:category[text()]?accept=xml"
                 var templateUrl_;

                 /**
                  * This is called if the update function has failed.
                  * @param {type} element
                  * @param {type} scope
                  * @param {Error} e
                  */
                 function failToLoadViews (element, scope, e) {
                     scope.config.view = undefined;
                     scope.control.viewMustBeInitialized = false;
                     scope.config.viewOk = false;

                     throw e || "Request error in selectView directive!";
                 }
                 /**
                  * Mets à jours la liste des vues en utilisant l'url de base de lavoisier donnée par l'utilisateur.
                  * @param {type} scope
                  * @param {type} element
                  * @param {type} attrs
                  */
                 function updateSelectView (scope, element, attrs) {
                     // Création de l'url de récupération des vues utilisateur.
                     var url_for_all_views_detail = scope.config.baseurl  + "/configuration"
                             + "/cfg:config/cfg:view[cfg:info/cfg:category!='Hidden'%20and%20cfg:info/cfg:category!='Configuration'%20and%20cfg:info/cfg:category!='Environment'%20and%20cfg:info/cfg:category!='Monitoring']/@name"
                             + "|/cfg:config/cfg:view[cfg:info/cfg:category!='Hidden'%20and%20cfg:info/cfg:category!='Configuration'%20and%20cfg:info/cfg:category!='Environment'%20and%20cfg:info/cfg:category!='Monitoring']/cfg:info/cfg:category[text()]"
                             + "|/cfg:config/cfg:view[not(cfg:info/cfg:category)]/@name?accept=xml";

                     lavoisierHttpRequestService.getResult({url : url_for_all_views_detail}).then(
                         function (value) {

                             scope.control.viewMustBeInitialized = true;

                             try {
                                 // Transformation de la réponse du serveur en json.
                                 var jsonEntries = parserService.xml_str2json(value);

                                 // Récupération des vues par catégories.
                                 var views = {};
                                 for (var e = 0 ; e < jsonEntries.entries.entry.length ; e++) {
                                     if (!views[jsonEntries.entries.category[e].__text]) views[jsonEntries.entries.category[e].__text] = [];
                                     if (scope.config.view === jsonEntries.entries.entry[e].__text) scope.control.viewMustBeInitialized = false;
                                     views[jsonEntries.entries.category[e].__text].push(jsonEntries.entries.entry[e].__text);
                                 }

                                 scope.views = [];
                                 // Ajout à scope.views des différentes vues avec : {category , name}
                                 for (var index in views) {
                                     for (var o = 0 ; o < views[index].length ; o++) {
                                         scope.views.push({category : index, name : views[index][o]});
                                     }
                                 }

                                 // Dernière vérification avant validation.
                                 if (scope.views.length === 0) throw "Empty views in selectView directive!";

                                 scope.config.viewOk = true;
                             } catch(e) {
                                 failToLoadViews(element, scope, e);
                             }
                         },
                         function () {
                             failToLoadViews(element, scope);
                         }
                     ).then(
                         function () {
                             if (scope.control.viewMustBeInitialized) {
                                 scope.config.view = scope.control.controller.defaultViewValue;
                                 scope.control.viewMustBeInitialized = false;
                             }

                             templateUrl_ = "templates/views_success.html";
                         },function () {
                             templateUrl_ = "templates/views_failed.html";
                         }
                     );
                 }

                 return {
                     scope : {
                         control : '='
                     },
                     link : function (scope, element, attrs) {
                         // scope.control = directive attribut control. Ici lavoisierFormEditCtrl.select_view -> ligne : 8.
                         scope.config = scope.control.config;
                         scope.control.controller.majViewContent = function () { updateSelectView(scope, element, attrs); };

                         if (!scope.control.controller.addEvent) {
                             angular.element(document.getElementById("lavoisier_url")).on("focusout", scope.control.controller.majViewContent);
                             scope.control.controller.addEvent = true;
                         }

                         updateSelectView(scope, element, attrs);

                         scope.getContentUrl  = function () {
                             if (templateUrl_)
                                 return LAVOISIER_TEMPLATES_PATH
                                     + "/edit/directives/" + templateUrl_;
                             else return false;

                         };
                     },
                     template : "<div ng-include='getContentUrl()'></div>"
                 };
             }
         ]
    ).directive('parametersView',
         [
             "lavoisierHttpRequestService", "parserService", "domService", "$compile",
             function (lavoisierHttpRequestService, parserService, domService, $compile) {
                 /**
                  * Donne les éléments de formulaire contenu dans l'élément donné.
                  * @param {Element} domElement
                  * @returns {Element}
                  */
                 function getFormElement (domElement) {
                     if (domElement.getElementsByTagName("select").length > 0) return domElement.getElementsByTagName("select")[0];
                     else if (domElement.getElementsByTagName("input").length > 0) return domElement.getElementsByTagName("input")[0];
                 }

                 function updateParametersView (scope, element, attrs) {
                     // Suppression des node fils de 'element'.
                     if (element.children().length > 0) element.empty();

                     // Préparation de la configuration pour acceptation des valeur données.
                     if (!scope.config["parameters_" + scope.config.view]) {
                         scope.config["parameters_" + scope.config.view] = {};
                     }
                     // Construction de l'url de récupération de la page HTML.
                     var url_for_parameters = scope.config.baseurl + "/form/" + scope.config.view;

                     lavoisierHttpRequestService.getResult({url : url_for_parameters, params : {accept : "html"}})
                         .then(function (value) {

                             // Convertion de la valeur de retour en HTML DOM.
                             var html = parserService.htmlDomFromString(value);
                             // Récupération du formulaire qui nous interresse.
                             var parameter_form = html.getElementsByClassName("content-wrapper")[0].getElementsByTagName("form")[0];
                             // Suppression des deux dernier node enfant du formulaire.
                             for (var i = 1 ; i <= 2 ; i++) parameter_form.removeChild(parameter_form.lastChild);
                             // Récupération de tous les div contenant les éléments de formulaire.
                             var div_form_group = parameter_form.getElementsByClassName("form-group");
                             // Récupération de tous les éléments de formulaire et conversion pour intégration.
                             for (var i_div = 0 ; i_div < div_form_group.length ; i_div++) {
                                 // Récupération de l'élément de formulaire.
                                 var formElement = getFormElement(div_form_group[i_div]);
                                 // Création du div qui va contenir l'élément de formulaire.
                                 var div = domService.createElement("div")
                                         .setAttribute("class", "form-group")
                                         .getDom();
                                 // Création et ajout du label.
                                 var label = domService.createElement("label").setAttribute("for", formElement.getAttribute("name")).getDom();
                                 label.innerHTML = formElement.getAttribute("name");
                                 div.appendChild(label);
                                 // Modification de l'élément de formulaire et ajout dans div.
                                 scope.config["parameters_" + scope.config.view][formElement.getAttribute("name")] = scope.config["parameters_" + scope.config.view][formElement.getAttribute("name")] || undefined;
                                 div.appendChild(
                                     domService.setDom(formElement)
                                         .setAttribute("class", "form-control")
                                         .setAttribute("ng-model", "config['parameters_" + scope.config.view + "']['" + formElement.getAttribute("name") + "']")
                                         .getDom());

                                 // Récupération de la valeur par défaut de l'élément s'il en a une.
                                 if (!scope.config["parameters_" + scope.config.view][formElement.getAttribute("name")]) {
                                     // Traitement différent selon si input ou select
                                     if (formElement.tagName === "input") {
                                         scope.config["parameters_" + scope.config.view][formElement.getAttribute("name")] = formElement.value || undefined;
                                     } else {
                                         var option_tab = formElement.getElementsByTagName("option");
                                         for (var o = 0 ; o < option_tab.length ; o++) {
                                             if (option_tab[o].getAttribute("selected") !== null) {
                                                 scope.config["parameters_" + scope.config.view][formElement.getAttribute("name")] = option_tab[o].value;
                                                 break;
                                             }
                                         }
                                     }
                                 }
                                 // Ajout des éléments dans l'élément de directive.
                                 element.append($compile(div)(scope));

                                 // Ajout ici de l'évènement car si on l'ajoute avant le deferred, on risque de l'ajouter avant que l'élément "#select_for_lavoisier_view" ne soit lui même ajouter dans le html.
                                 if (!scope.control.controller.eventAdded) {
                                     // Ajout d'un évènement sur l'élément select des vues.
                                     angular.element(document.getElementById("select_for_lavoisier_view")).on("change", scope.control.controller.majParametersContent);
                                     // Pour n'ajouter l'évènement qu'une foi.
                                     scope.control.controller.eventAdded = true;
                                 }
                             }
                         });
                 }

                 return {
                     scope : {
                         control : '='
                     },
                     link : function (scope, element, attrs) {
                         // scope.control = directiver attribut control. Ici lavoisierFormEditCtrl.parameters_view -> ligne : 14.
                         scope.config = scope.control.config;
                         scope.control.controller.majParametersContent = function () { updateParametersView(scope, element, attrs); };
                         scope.control.controller.majParametersContent();
                     }
                 };
             }
         ]
    );